//+------------------------------------------------------------------+
//|                                          NonLinearRegression.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      ""

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1 Green
#property indicator_color2 Orange
#property indicator_color3 Orange
#property indicator_width1 2
#property indicator_width2 2
#property indicator_width3 2

//
//
//
//
//
extern string TimeFrame        = "Current time frame";
extern int    NLRLength        = 20;
extern int    NLRPrice         = PRICE_CLOSE;
extern bool   Interpolate      = true;
extern bool   MultiColor       = true;

extern string  _               = "alerts settings";
extern bool    alertsOn        = true;
extern bool    alertsOnCurrent = false;
extern bool    alertsMessage   = true;
extern bool    alertsSound     = true;
extern bool    alertsEmail     = true;


//
//
//
//
//

double nlrBuffer[];
double ndaBuffer[];
double ndbBuffer[];
double trnBuffer[];


int    timeFrame;
string IndicatorFileName;
bool   ReturningBars;
bool   CalculatingNlr;


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int init()
{
   IndicatorBuffers(4);
   SetIndexBuffer(0,nlrBuffer);
   SetIndexBuffer(1,ndaBuffer);
   SetIndexBuffer(2,ndbBuffer);
   SetIndexBuffer(3,trnBuffer);
   
   
   IndicatorFileName = WindowExpertName();
   CalculatingNlr    = (TimeFrame=="CalculateNlr"); if (CalculatingNlr) return(0);
   ReturningBars     = (TimeFrame=="returnBars");    if (ReturningBars)  return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
   
   //
   //
   //
   //
   //
   
   NLRLength = MathMax(MathMin(NLRLength,500),2);
   
   
   
   IndicatorShortName("NonLinearRegression "+timeFrameToString(timeFrame));    
   return(0);
}


int deinit() { return(0); }


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int start()
{
   int    counted_bars=IndicatorCounted();
   int    limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars) { nlrBuffer[0] = limit+1; return(0); }
           
   if (CalculatingNlr || timeFrame==Period())
   {     

   //
   //
   //
   //
   //

   if (MultiColor && !CalculatingNlr && trnBuffer[limit] == -1) CleanPoint(limit,ndaBuffer,ndbBuffer);
      for(int i=limit; i>=0; i--)
      {
         nlrBuffer[i] = NLR(NLRLength, NLRPrice, i, 0);
         ndaBuffer[i] = EMPTY_VALUE;
         ndbBuffer[i] = EMPTY_VALUE;
         trnBuffer[i] = trnBuffer[i+1];
      
         //
         //
         //
         //
         //
            
         if (nlrBuffer[i] > nlrBuffer[i+1]) trnBuffer[i] =  1;
         if (nlrBuffer[i] < nlrBuffer[i+1]) trnBuffer[i] = -1;
            if (MultiColor && !CalculatingNlr && trnBuffer[i] == -1) PlotPoint(i,ndaBuffer,ndbBuffer,nlrBuffer);
            manageAlerts();
      }
   return(0);
}

  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

//
   //
   //
   //
   //
   
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (MultiColor && trnBuffer[limit] == -1) CleanPoint(limit,ndaBuffer,ndbBuffer);

   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         trnBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateNlr",NLRLength,NLRPrice,3,y);
         nlrBuffer[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateNlr",NLRLength,NLRPrice,0,y);
         ndaBuffer[i] = EMPTY_VALUE;
         ndaBuffer[i] = EMPTY_VALUE;

         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++) nlrBuffer[i+k] = k*factor*nlrBuffer[i+n] + (1.0-k*factor)*nlrBuffer[i];
   }
   if (MultiColor) for (i=limit;i>=0;i--) if (trnBuffer[i] == -1) PlotPoint(i,ndaBuffer,ndbBuffer,nlrBuffer);
   manageAlerts();
   //
   //
   //
   //
   //
   return(0);
}


string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}




void manageAlerts()
{
   if (!CalculatingNlr && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trnBuffer[whichBar] != trnBuffer[whichBar+1])
      {
         if (trnBuffer[whichBar] ==  1) doAlert(whichBar,"up");
         if (trnBuffer[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," NonLinearRegression changed direction to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," NonLinearRegression "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}


double YValue[];
double XValue[];
double XM,YM,XM2;
int    arraySize=-1;

//
//
//
//
//

void setArrays(int Length)
{
   arraySize = Length;
      ArrayResize(YValue,Length);
      ArrayResize(XValue,Length);
}

//
//
//
//
//

double NLR(int Length,int AppliedPrice,int shift,int desiredBar)
{
   Length = MathMin(Length,Bars-shift);
      if (Length<1) return(EMPTY_VALUE);
      if (arraySize!=Length) setArrays(Length);
   
   //
   //
   //
   //
   //
   
   double AvgX = 0.0;
   double AvgY = 0.0;

   for(int i=0;i<Length;i++)
      {
         XValue[i] = i;
         YValue[i] = iMA(NULL,0,1,0,MODE_SMA,AppliedPrice,shift+i);
            AvgX  += XValue[i];
            AvgY  += YValue[i];
      }
      AvgX /= Length;
      AvgY /= Length;

      //
      //
      //
      //
      //
      
      double SXX   = 0;
      double SXY   = 0;
      double SYY   = 0;
      double SXX2  = 0;
      double SX2X2 = 0;
      double SYX2  = 0;

      for(i=0;i<Length;i++)
      {
         XM    = XValue[i] - AvgX; 
         YM    = YValue[i] - AvgY;
         XM2   = XValue[i] * XValue[i] - AvgX*AvgX;
         SXX   += XM*XM;
         SXY   += XM*YM;
         SYY   += YM*YM;
         SXX2  += XM*XM2;
         SX2X2 += XM2*XM2;
         SYX2  += YM*XM2;
      }         

      //
      //
      //
      //
      //
      
      double tmp;
      double ACoeff=0;
      double BCoeff=0;
      double CCoeff=0;

      tmp = SXX*SX2X2 - SXX2*SXX2;
      if (tmp!=0) {
         BCoeff = ( SXY*SX2X2 - SYX2*SXX2 ) / tmp;
         CCoeff = ( SXX*SYX2  - SXX2*SXY )  / tmp;
      }
      ACoeff = AvgY   - BCoeff*AvgX       - CCoeff*AvgX*AvgX;
      tmp    = ACoeff + BCoeff*desiredBar + CCoeff*desiredBar*desiredBar;

   //
   //
   //
   //
   //
   
   return(tmp);      
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
      if (first[i+2] == EMPTY_VALUE) {
          first[i]    = from[i];
          first[i+1]  = from[i+1];
          second[i]   = EMPTY_VALUE;
         }
      else {
          second[i]   = from[i];
          second[i+1] = from[i+1];
          first[i]    = EMPTY_VALUE;
         }
      }
   else
      {
         first[i]   = from[i];
         second[i]  = EMPTY_VALUE;
      }
}